# $Id: Global.pm,v 1.2 2006/11/29 07:08:22 cl Exp $

##
# The Global class acts as a singleton and provides controlled access to global
#	data and related methods.
#


use strict;
use warnings;


package Bio::IScan::Global;
#	our @ISA = '';


# static
our $Global = __PACKAGE__->new();


sub Global { # static
	return $Global;
}


sub Global::Add_Debug_Pattern { # static
	$Global->add_debug_pattern(@_);
}


sub Global::Debug { # static
	$Global->debug(@_);
}


sub Global::Progress { # static
	return $Global->progress(@_);
}


sub Global::Show_Progress { # static
	return $Global->show_progress(@_);
}


sub Global::Configuration_Parameter { # static
	return $Global->configuration_parameter(@_);
}


sub new { # constructor
	my $class = shift;
	my $this = {};
	
	$this->{'debug_patterns'} = [];
	$this->{'configuration_parameters'} = {};

	return bless $this, $class;
}


sub add_debug_pattern { # instance
	my $this = shift;
	my $pattern = shift;
	push @{ $this->{'debug_patterns'} }, $pattern;
}


sub debug { # instance
	my $this = shift;
	my $package = shift;
	
	for my $pattern (@{ $this->{'debug_patterns'} }) {
		if ($package =~ /$pattern/) {
			print STDERR @_, "\n";
		}
	}
}


sub progress { # instance
	my $this = shift;
	
	if ($this->show_progress()) {
		print(STDERR @_);
	}
}


sub show_progress { # instance
	my $this = shift;
	
	if (@_) { $this->{'progress'} = shift; }
	
	return $this->{'progress'};
}


sub configuration_parameter { # instance
	my $this = shift;
	my $name = shift;
	my $value = shift;
	
	if (defined $value) {
		$this->{'configuration_parameters'}->{$name} = $value;
	}
	
	return $this->{'configuration_parameters'}->{$name};
}


1;

__END__
