# $Id: Search_Request.pm,v 1.11 2007/05/28 03:53:00 cl Exp $

##
# A Search_Request packages all the request parameters submitted to IScan.
#


use strict;
use warnings;

use Bio::Root::Root;

use Bio::IScan::Utilities;


package Bio::IScan::Search_Request;
	our @ISA = 'Bio::Root::Root';


sub new { # constructor
	my $class = shift;
	my $this = $class->SUPER::new(@_);
	
	$this->{'parameters'} = [ 
		'maximum_orf_distance',
		'duplicate_hsp_overlap',
		'group_hsps_accross_strands',
		'blast_maximum_expect',
		'blast_minimum_hsp_fraction_identical',
		'blast_minimum_query_alignment_fraction_identical',		
		'ir_expand_out',
		'ir_expand_in',
		'dr_expand_out',
		'dr_expand_in',
		'alignment_match_value',
		'alignment_mismatch_value',
		'alignment_gap_extension_value' 
	];
	$this->{'optional_parameters'} = { 
		'search_name' => undef,
		'queries' => undef,, 
		'ir_statistics_iterations' => undef,
		'ir_left_profile' => undef,
		'ir_right_profile' => undef
	};

	my $arguments = Utilities::Process_Arguments($this->{'parameters'},
												 $this->{'optional_prameters'},
												 @_);
	map { $this->{$_} = $arguments->{$_} } keys %{ $arguments };
		
	return bless $this, $class;	
}


sub search_name { #
	my $this = shift;
	
	if (@_) {
		$this->{'search_name'} = shift;
	}
	
	return $this->{'search_name'};	
}


sub queries { # instance
	my $this = shift;
	
	if (@_) {
		$this->{queries} = shift;
	}
	
	return $this->{queries};
}


sub maximum_orf_distance { # instnace
	my $this = shift;

	if (@_) {
		$this->{maximum_orf_distance} = shift;
	}
	
	return $this->{maximum_orf_distance};
}


sub duplicate_hsp_overlap { # instance
	my $this = shift;
	
	if (@_) {
		$this->{'duplicate_hsp_overlap'} = shift;
	}
	
	return $this->{'duplicate_hsp_overlap'};
}


sub group_hsps_accross_strands { # instance
	my $this = shift;
	
	if (@_) {
		$this->{'group_hsps_accross_strands'} = shift;
	}
	
	return $this->{'group_hsps_accross_strands'}
}


sub ir_expand_out { # instance
	my $this = shift;
	
	if (@_) {
		$this->{ir_expand_out} = shift;
	}
	
	return $this->{ir_expand_out};
}


sub ir_expand_in { # instance
	my $this = shift;
	
	if (@_) {
		$this->{ir_expand_in} = shift;
	}
	
	return $this->{ir_expand_in};
}


sub ir_left_profile { # instance
	my $this = shift;
	
	if (@_) {
		$this->{'ir_left_profile'} = shift;
	}
	
	return $this->{'ir_left_profile'};
}


sub ir_right_profile { # instance
	my $this = shift;
	
	if (@_) {
		$this->{'ir_right_profile'} = shift;
	}
	
	return $this->{'ir_right_profile'};
}


sub dr_expand_out { # instance
	my $this = shift;
	
	if (@_) {
		$this->{dr_expand_out} = shift;
	}
	
	return $this->{dr_expand_out};
}


sub dr_expand_in { # instance
	my $this = shift;
	
	if (@_) {
		$this->{dr_expand_in} = shift;
	}
	
	return $this->{dr_expand_in};
}


sub blast_maximum_expect { # instance
	my $this = shift;
	
	if (@_) {
		$this->{blast_maximum_expect} = shift;
	}
	
	return $this->{blast_maximum_expect};
}


sub blast_minimum_hsp_fraction_identical { # instance
	my $this = shift;
	
	if (@_) {
		$this->{'blast_minimum_hsp_fraction_identical'} = shift;
	}
	
	return $this->{'blast_minimum_hsp_fraction_identical'};
}


sub blast_minimum_query_alignment_fraction_identical { # instance
	my $this = shift;
	
	if (@_) {
		$this->{'blast_minimum_query_alignment_fraction_identical'} = shift;
	}
	
	return $this->{'blast_minimum_query_alignment_fraction_identical'};
}


sub ir_generate_statistics { # instance
	my $this = shift;

	if (@_) {
		$this->throw(-class => 'Bio::Root::BadParameter',
					 -text => "Attribute can not be set: ir_generate_statistics",
					 -value => 'ir_generate_statistics');
	}

	if ($this->ir_statistics_iterations() > 0) {
		return 1;
	}

	return 0;
}


sub ir_statistics_iterations { # instance
	my $this = shift;
	
	if (@_) {
		$this->{ir_statistics_iterations} = shift;
	}
	
	if (defined $this->{ir_statistics_iterations}) {
		return $this->{ir_statistics_iterations};
	}
	
	return 0;
}


sub alignment_match_value { # instance
	my $this = shift;
	
	if (@_) {
		$this->{alignment_match_value} = shift;
	}
	
	return $this->{alignment_match_value};
}


sub alignment_mismatch_value { # instance
	my $this = shift;
	
	if (@_) {
		$this->{alignment_mismatch_value} = shift;
	}
	
	return $this->{alignment_mismatch_value};
}


sub alignment_gap_extension_value { # instance
	my $this = shift;
	
	if (@_) {
		$this->{alignment_gap_extension_value} = shift;
	}
	
	return $this->{alignment_gap_extension_value};
}


sub context_parameter { # instance
	my $this = shift;
	my $name = shift;
	my $value = shift;
	
	if (defined $value) {
		$this->{'context_parameter'}->{$name} = $value;
	}
	
	return $this->{'context_parameter'}->{$name};
}

sub remove_context_parameter { # instance
	my $this = shift;
	my $name = shift;

	my $value = $this->{'context_parameter'}->{$name};
	delete($this->{'context_parameter'}->{$name});

	return $value;
}


sub get_parameter_names { # instance
	my $parameter_names = [
		'maximum_orf_distance',
		'duplicate_hsp_overlap',
		'group_hsps_accross_strands',
		'blast_maximum_expect',
		'blast_minimum_hsp_fraction_identical',
		'blast_minimum_query_alignment_fraction_identical',	
		'ir_expand_out',
		'ir_expand_in',
		'dr_expand_out',
		'dr_expand_in',
		'alignment_match_value',
		'alignment_mismatch_value',
		'alignment_gap_extension_value',
		#'search_name',
		#'queries', 
		'ir_statistics_iterations'
		#'ir_upstream_profile',
		#'ir_downstream_profile' 
		];
	
	return $parameter_names;
}


sub get_parameters { # instance
	my $this = shift;

	my $parameters = {};
	map({ $parameters->{$_} = $this->{$_} } @{ $this->get_parameter_names() });
	
	return $parameters;
}


sub Search_Request::Get_Parameter_Description { # static
	return Get_Parameter_Description(@_);
}


sub Get_Parameter_Description { # static
	my $parameter = shift;
	
	if ($parameter eq 'blast_maximum_expect') {
		return "BLAST maximum expect";
	}
	elsif ($parameter eq 'blast_minimum_hsp_fraction_identical') {
		return "BLAST minimum HSP fraction identical";
	}
	elsif ($parameter eq 'blast_minimum_query_alignment_fraction_identical') {
		return "BLAST minimum query-alignment fraction identical";
	}
	elsif ($parameter eq 'maximum_orf_distance') {
		return "Maximum ORF distance";
	}
	elsif ($parameter eq 'duplicate_hsp_overlap') {
		return "Duplicate HSP overlap";
	}
	elsif ($parameter eq 'group_hsps_accross_strands') {
		return "Group HSPs accross strands";
	}
	elsif ($parameter eq 'ir_expand_out') {
		return "Inverted repeat outwards expansion";
	}
	elsif ($parameter eq 'ir_expand_in') {
		return "Inverted repeat inwards expansion";
	}
	elsif ($parameter eq 'dr_expand_out') {
		return "Direct repeat outwards expansion";
	}
	elsif ($parameter eq 'dr_expand_in') {
		return "Direct repeat inwards expansion";
	}
	elsif ($parameter eq 'alignment_match_value') {
		return "Alignment match value";
	}
	elsif ($parameter eq 'alignment_mismatch_value') {
		return "Alignment mismatch penalty";
	}
	elsif ($parameter eq 'alignment_gap_extension_value') {
		return "Alignment gap or gap extension penalty";
	}
	elsif ($parameter eq 'ir_statistics_iterations') {
		return "Inverted repeat statistical iterations";
	}
	else {
		return "[Unknown]";
	}
}


@Search_Request_Exception::ISA = 'Bio::Root::Exception';


1;

__END__
