# $Id: Controller.pm,v 1.3 2006/11/29 07:04:12 cl Exp $

# POD documentation at the end of file


use strict;

use Bio::Root::Root;

use File::Spec;


package Bio::IScan::Server::Controller;
	our @ISA = 'Bio::Root::Root';


sub new { # instance
	my $class = shift;
	my $this = $class->SUPER::new(@_);
	
	my $parameters = [];
	my $optional_parameters = ['keep_files', 'temporary_path'];
	
	my $arguments = $this->_process_arguments($parameters, $optional_parameters, @_);
	map { $this->{$_} = $arguments->{$_}; } keys %{ $arguments };
	
	if (!defined $this->{'keep_files'}) {
		$this->{'keep_files'} = 0;
	}
	
	if (!defined $this->{'temporary_path'}) {
		$this->{'temporary_path'} = File::Spec->tmpdir();
	}

	return bless $this, $class;
	
}


sub temporary_file { # instance
	my $this = shift;
	
	my $template = shift;
	my $suffix = shift;
	
	my $file_handle = undef;
	my $file_name = undef;
	

	my $file_entry = '_file_' . $suffix;
	my $unlink = ($this->{'keep_files'}) ? 0 : 1;
	
	if (!$this->{'keep_files'}) {
		if (defined $this->{$file_entry}) {
			if (!(open $file_handle, '>', $this->{$file_entry})) {
				$this->throw(-class => 'BLAST_Controller_Exception',
							 -text => "Unable to open temporary file: " . 
								$this->{file_entry},
							 -value => $!);
			}
			return ($file_handle, $this->{$file_entry});
		}
	}	
	
	eval {
		($file_handle, $file_name) = 
			File::Temp->tempfile($template, 
								 DIR => $this->{'temporary_path'},
								 SUFFIX => $suffix, 
								 UNLINK => $unlink);
	};
	if ($@) {
		$this->throw(-class => 'Controller_Exception',
					 -text => "Unable to create temporary file: $!",
					 -value => $!);
	}
	
	# Keep the '$file_name' to be used later	
	if (!$this->{'keep_files'}) {
		$this->{$file_entry} = $file_name;
	}		
	
	return ($file_handle, $file_name);
}


sub _process_arguments { # instance
	my $this = shift;
	
	my $parameters = shift;
	my $optional_parameters = shift;
	my ($arguments) = @_;
	
	
	# Convert arguments to lowercase
	%{ $arguments } = map { lc($_) => $arguments->{$_} } keys %{ $arguments };
	
	# Make sure all the required parameters have been specified, and tranfer
	#   them over to class instance storage
	for my $test (@{ $parameters} ) {
		if (!defined $arguments->{$test}) {
			$this->throw(-class => 'Bio::Root::BadParameter',
						 -text => "Argument must be specified: $test",
						 -value => $test);
		}		
	}
	
	return $arguments;
}


##
# Exception class for Controller.
#
@Controller_Exception::ISA = 'Bio::Root::Exception';

1;
