# $Id: Utilities.pm,v 1.4 2006/12/27 02:28:00 cl Exp $


use strict;
use warnings;

use Bio::Root::Root;


package Bio::IScan::Utilities;


sub Utilities::Parse_FastA_Header { # static
	return Bio::IScan::Utilities::Parse_FastA_Header(@_);
}


sub Utilities::Process_Arguments { # static
	# It is not clear why the below does not work.	
	#__PACKAGE__::Process_Arguments(@_);	
	
	Bio::IScan::Utilities::Process_Arguments(@_);
}


sub Parse_FastA_Header { # static
	my $header_line = shift;
		
	my $id = Bio::IScan::FastA_Header->new({ 'header_line' => $header_line });

	return $id;
}


sub Process_Arguments { # static
	my $parameters = shift;
	my $optional_parameters = shift;
	my ($arguments) = @_;
	
	
	# Convert arguments to lowercase
	%{ $arguments } = map { lc($_) => $arguments->{$_} } keys %{ $arguments };
	
	# If the special copy constructor <_copy_> argument is provided, set each
	#	argument to the value provided by the <_copy_> argument.
	if (exists $arguments->{'_copy_'}) {
		for my $attribute (@{ $parameters }) {
			if (!defined $arguments->{$attribute}) {
				$arguments->{$attribute} = $arguments->{'_copy_'}->{$attribute};
			}
		}
		for my $attribute (keys %{ $optional_parameters }) {
			if (!defined $arguments->{$attribute}) {
				$arguments->{$attribute} = $arguments->{'_copy_'}->{$attribute};
			}
		}
	}
	else {
		# Make sure all the required parameters have been specified
		for my $test (@{ $parameters } ) {
			if (!exists $arguments->{$test}) {
				my $root = Bio::Root::Root->new();
				$root->throw(-class => 'Bio::Root::BadParameter',
							 -text => "Argument must be specified: $test",
							 -value => $test);
			}
		}
		
		# Set default values for optional parameters 
		for my $test (keys %{ $optional_parameters }) {
			if (!exists $arguments->{$test}) {
				$arguments->{$test} = $optional_parameters->{$test};
			}
		}
	}	
	
	return $arguments;
}


package Bio::IScan::FastA_Header;

sub new { # constructor
	my $class = shift;
	my $this = {};

	my $parameters = [ 'header_line' ];
	my $optional_parameters = {};

	my $arguments = Utilities::Process_Arguments($parameters, 
												 $optional_parameters,
												 @_);
	
	# ID format is <tag>|<accession>.<version>|<locus> <source>
	$arguments->{'header_line'} =~ /([^\|]*)\|([^\|]*)\|([^\s]*)\s*(.*)/;
	$arguments->{'tag'} = $1;
	$arguments->{'accession'} = $2;
	$arguments->{'locus'} = $3;
	$arguments->{'source'} = $4;

	$arguments->{'version'} = (split(/\./, $arguments->{'accession'}))[1];

	map { $this->{$_} = $arguments->{$_} } keys %{ $arguments };
	
	return bless $this, $class;
}

sub tag { # instance
	my $this = shift;
	
	if (@_) {
		$this->{'tag'} = shift;
	}
	
	return $this->{'tag'};
}

sub accession { # instance
	my $this = shift;
	
	if (@_) {
		$this->{'accession'} = shift;
	}
	
	return $this->{'accession'};
}

sub version { # instance
	my $this = shift;
	
	if (@_) {
		$this->{'version'} = shift;
	}
	
	return $this->{'version'};
}

sub locus { # instance
	my $this = shift;
	
	if (@_) {
		$this->{'locus'} = shift;
	}
	
	return $this->{'locus'};
}

sub source { # instance
	my $this = shift;
	
	if (@_) {
		$this->{'source'} = shift;
	}
	
	return $this->{'source'};	
}

sub header_string { # instance
	my $this = shift;
	my $id_separator = shift || "|";
	my $source_separator = shift || " ";
	
	return $this->{'tag'} . $id_separator . 
		   $this->{'accession'} . $id_separator . 
		   $this->{'locus'} . $source_separator . 
		   $this->{'source'};
}


1;
